@@ -19,6 +19,7 @@
  */
 
 #include "box_data.h"
+#include "../data_structure/glomconversions.h"
 #include "config.h"
 #include <libintl.h>
 
@@ -56,15 +57,15 @@ Glib::ustring Box_Data::get_WhereClause() const
     const Field& field = get_Entered_Field(i);
     Glib::ustring strClausePart;
 
-    const Glib::ustring strData = field.get_data();
-    if(strData.size())
+    const Gnome::Gda::Value data = field.get_data();
+    if(!GlomConversions::value_is_empty(data))
     {
       Gnome::Gda::FieldAttributes fieldInfo = field.get_field_info();
       //TODO: "table.name"
-      strClausePart = fieldInfo.get_name() + " LIKE " +  field.sql("%" + strData + "%"); //% is mysql wildcard for 0 or more characters.
+      strClausePart = fieldInfo.get_name() + " LIKE " +  field.sql(data); //% is mysql wildcard for 0 or more characters.
     }
 
-    if(strClausePart.size())
+    if(!strClausePart.empty())
       strClause += strClausePart + " ";
   }
 
@@ -94,9 +95,9 @@ bool Box_Data::record_new_from_entered()
     Gnome::Gda::FieldAttributes fieldInfo = field.get_field_info();
     Glib::ustring strFieldValue = field.sql(field.get_data());
 
-    if(strFieldValue.size())
+    if(!strFieldValue.empty())
     {
-      if(strNames.size())
+      if(!strNames.empty())
       {
         strNames += ", ";
         strValues += ", ";
@@ -108,7 +109,7 @@ bool Box_Data::record_new_from_entered()
   }
 
   //Put it all together to create the record with these field values:
-  if(strNames.size() && strValues.size())
+  if(!strNames.empty() && !strValues.empty())
   {
     Glib::ustring strQuery = "INSERT INTO " + m_strTableName + " (" + strNames + ") VALUES (" + strValues + ")";
     return Query_execute(strQuery);
@@ -183,8 +184,8 @@ Box_Data::type_vecFields Box_Data::get_table_fields_to_show(const Glib::ustring&
   const Document_Glom* pDoc = dynamic_cast<const Document_Glom*>(get_document());
   if(pDoc)
   {
-    Document_Glom::type_mapFieldSequence mapFieldSequence =  pDoc->get_data_layout_plus_new_fields(m_layout_name, table_name);
-    if(mapFieldSequence.empty())
+    Document_Glom::type_mapLayoutGroupSequence mapGroupSequence =  pDoc->get_data_layout_groups_plus_new_fields(m_layout_name, table_name);
+    if(mapGroupSequence.empty())
     {
       //No field sequence has been saved in the document, so we use all fields by default, so we start with something visible:
    
@@ -212,20 +213,26 @@ Box_Data::type_vecFields Box_Data::get_table_fields_to_show(const Glib::ustring&
       type_vecFields vecFieldsInDocument = pDoc->get_table_fields(table_name);
       
       //We will show the fields that the document says we should:
-      for(Document_Glom::type_mapFieldSequence::const_iterator iter = mapFieldSequence.begin(); iter != mapFieldSequence.end(); ++iter)
+      for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = mapGroupSequence.begin(); iter != mapGroupSequence.end(); ++iter)
       {
-        LayoutItem item = iter->second;
+        const LayoutGroup& group = iter->second;
 
-        if(!item.m_hidden)
+        if(true) //!group.m_hidden)
         {
-           //Get the field info:
-          Glib::ustring field_name = item.m_field_name;
-          type_vecFields::const_iterator iterFind = std::find_if(all_fields.begin(), all_fields.end(), predicate_FieldHasName<Field>(field_name));
-
-          //If the field does not exist anymore then we won't try to show it:
-          if(iterFind != all_fields.end() )
+          //Get the fields:
+          for(LayoutGroup::type_map_items::const_iterator iterItems = group.m_map_items.begin(); iterItems != group.m_map_items.end(); ++iterItems)
           {
-             result.push_back(*iterFind);
+            const LayoutItem& item = iterItems->second;
+            
+            //Get the field info:
+            Glib::ustring field_name = item.m_field_name;
+            type_vecFields::const_iterator iterFind = std::find_if(all_fields.begin(), all_fields.end(), predicate_FieldHasName<Field>(field_name));
+
+            //If the field does not exist anymore then we won't try to show it:
+            if(iterFind != all_fields.end() )
+            {
+               result.push_back(*iterFind);
+            }
           }
         }
         
@@ -257,6 +264,68 @@ guint Box_Data::generate_next_auto_increment(const Glib::ustring& table_name, co
   return result;
 }
 
+/** Get the fields whose values should be looked up when @a field_name changes, with
+ * the relationship used to lookup the value.
+ */
+Box_Data::type_list_lookups Box_Data::get_lookup_fields(const Glib::ustring& field_name) const
+{
+  type_list_lookups result;
+
+  const Document_Glom* document = dynamic_cast<const Document_Glom*>(get_document());
+  if(document)
+  {
+    for(type_vecFields::const_iterator iter = m_Fields.begin(); iter != m_Fields.end();  ++iter)
+    {
+      //Examine each field that looks up its data from a relationship:
+      if(iter->get_is_lookup())
+      {
+        //Get the relationship information:
+        Relationship relationship;
+        bool test = document->get_relationship(m_strTableName, iter->get_lookup_relationship(), relationship);
+        if(test)
+        {
+          //If the relationship is triggererd by the specified field:
+          if(relationship.get_from_field() == field_name)
+          {
+            //Add it:
+            result.push_back( type_pairFieldTrigger(*iter, relationship) );
+          }
+        }
+      }
+    }
+  }
+
+  return result;
+}
+
+Gnome::Gda::Value Box_Data::get_lookup_value(const Relationship& relationship, const Field& source_field, const Gnome::Gda::Value& key_value)
+{
+  Gnome::Gda::Value result;
+  
+  Field to_key_field;
+  bool test = get_fields_for_table_one_field(relationship.get_to_table(), relationship.get_to_field(), to_key_field);
+  if(test)
+  {
+    Glib::ustring strQuery = "SELECT " + relationship.get_to_table() + "." + source_field.get_name() + " FROM " +  relationship.get_to_table();
+    strQuery += " WHERE " + relationship.get_to_field() + " = " + to_key_field.sql(key_value);
+
+    Glib::RefPtr<Gnome::Gda::DataModel> data_model = Query_execute(strQuery);
+    if(data_model && data_model->get_n_rows())
+    {
+      //There should be only 1 row. Well, there could be more but we will ignore them.
+      result = data_model->get_value_at(0, 0);
+    }
+    else
+    {
+      handle_error();
+    }
+  }
+
+  return result;
+}
+  
+
+
 
   
   
