@@ -38,9 +38,9 @@ void Box_DB_Table_Relationships::init()
   pack_start(m_AddDel);
   m_colName = m_AddDel.add_column(gettext("Name"));
 
-  m_colFromField = m_AddDel.add_column(gettext("Field"), AddDelColumnInfo::STYLE_Choices);
+  m_colFromField = m_AddDel.add_column(gettext("From Field"), AddDelColumnInfo::STYLE_Choices);
   m_colToTable = m_AddDel.add_column(gettext("Table"), AddDelColumnInfo::STYLE_Choices);
-  m_colToField = m_AddDel.add_column(gettext("Field"), AddDelColumnInfo::STYLE_Choices);
+  m_colToField = m_AddDel.add_column(gettext("To Field"), AddDelColumnInfo::STYLE_Choices);
 
   //Connect signals:
   m_AddDel.signal_user_activated().connect(sigc::mem_fun(*this, &Box_DB_Table_Relationships::on_AddDel_user_activated));
@@ -84,17 +84,18 @@ void Box_DB_Table_Relationships::fill_from_database()
        const Relationship& relationship = *iter;
 
        //Name:
-       guint uiRow = m_AddDel.add_item(relationship.get_name());
-
+       Gtk::TreeModel::iterator iterTree = m_AddDel.add_item(relationship.get_name());
+       m_AddDel.set_value(iterTree, m_colName, relationship.get_name());
+       
        //From Field:
-       m_AddDel.set_value(uiRow, m_colFromField, relationship.get_from_field());
+       m_AddDel.set_value(iterTree, m_colFromField, relationship.get_from_field());
 
        //To Table:
        const Glib::ustring& strToTable = relationship.get_to_table();
-       m_AddDel.set_value(uiRow, m_colToTable, strToTable);
+       m_AddDel.set_value(iterTree, m_colToTable, strToTable);
 
        //To Field:
-       m_AddDel.set_value(uiRow, m_colToField, relationship.get_to_field());
+       m_AddDel.set_value(iterTree, m_colToField, relationship.get_to_field());
 
     }
   }
@@ -103,20 +104,24 @@ void Box_DB_Table_Relationships::fill_from_database()
 }
 
 void Box_DB_Table_Relationships::save_to_document()
-{
+{ 
   //Build relationships from AddDel:
   Document_Glom::type_vecRelationships vecRelationships;
 
-  for(guint i = 0; i < m_AddDel.get_count(); i++)
+  for(Gtk::TreeModel::iterator iter = m_AddDel.get_model()->children().begin(); iter != m_AddDel.get_model()->children().end(); ++iter)
   {
-    Relationship relationship;
-    relationship.set_name(m_AddDel.get_value(i, m_colName));
-    relationship.set_from_table(m_strTableName);
-    relationship.set_from_field(m_AddDel.get_value(i, m_colFromField));
-    relationship.set_to_table(m_AddDel.get_value(i, m_colToTable));
-    relationship.set_to_field(m_AddDel.get_value(i, m_colToField));
+    const Glib::ustring name = m_AddDel.get_value(iter, m_colName);
+    if(!name.empty())
+    {
+      Relationship relationship;
+      relationship.set_name(name);
+      relationship.set_from_table(m_strTableName);
+      relationship.set_from_field(m_AddDel.get_value(iter, m_colFromField));
+      relationship.set_to_table(m_AddDel.get_value(iter, m_colToTable));
+      relationship.set_to_field(m_AddDel.get_value(iter, m_colToField));
 
-    vecRelationships.push_back(relationship);
+      vecRelationships.push_back(relationship);
+    }
   }
 
   //Update the Document with these relationships.
@@ -126,22 +131,16 @@ void Box_DB_Table_Relationships::save_to_document()
   Box_DB_Table::save_to_document();
 }
 
-Box_DB_Table_Relationships::type_vecStrings Box_DB_Table_Relationships::util_vecStrings_from_Fields(const type_vecFields& fields)
+void Box_DB_Table_Relationships::on_AddDel_user_changed(const Gtk::TreeModel::iterator& row, guint col)
 {
-  //Get vector of field names, suitable for a combo box:
-
-  type_vecStrings vecNames;
-  for(type_vecFields::size_type i = 0; i < fields.size(); i++)
+  if(col == m_colName)
   {
-    vecNames.push_back(fields[i].get_name());
+    //The name is the key, so If the name has been changed then the key must change too.
+    Glib::ustring new_name = m_AddDel.get_value(row, m_colName);
+    if(!new_name.empty())
+      m_AddDel.set_value_key(row, new_name);
   }
-
-  return vecNames;
-}
-
-void Box_DB_Table_Relationships::on_AddDel_user_changed(guint row, guint col)
-{
-  if(col == m_colToTable)
+  else if(col == m_colToTable)
   {
     //User chose a new table so we need to wipe the field
     //because it might not be a valid field for that table:
@@ -151,15 +150,18 @@ void Box_DB_Table_Relationships::on_AddDel_user_changed(guint row, guint col)
   set_modified();
 }
 
-void Box_DB_Table_Relationships::on_AddDel_user_activated(guint row, guint col)
+void Box_DB_Table_Relationships::on_AddDel_user_activated(const Gtk::TreeModel::iterator& row, guint col)
 {
+
+ g_warning("on_AddDel_user_activated setting to field col=%d", col);
+ 
   if(col == m_colToField)
   {
     Bakery::BusyCursor(*get_app_window());
         
-    const Glib::ustring& strTableName = m_AddDel.get_value(row, m_colToTable);
+    const Glib::ustring table_name = m_AddDel.get_value(row, m_colToTable);
 
-    if(strTableName.size())
+    if(!table_name.empty())
     {
       //Set list of 'To' fields depending on table:
       m_AddDel.set_value(row, m_colToField, Glib::ustring(""));
@@ -169,12 +171,13 @@ void Box_DB_Table_Relationships::on_AddDel_user_activated(guint row, guint col)
       {
         Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
      
-        type_vecStrings vecFields = util_vecStrings_from_Fields(get_fields_for_table(strTableName));
+        type_vecStrings vecFields = util_vecStrings_from_Fields(get_fields_for_table(table_name));
 
         //This would cause a lot of tedious re-filling:
         //m_AddDel.set_column_choices(m_colToField, vecFields);
         //fill_from_database();
 
+        g_warning("setting to field col=%d, choices.size=%d", m_colToField, vecFields.size());
         m_AddDel.set_column_choices(m_colToField, vecFields);
       }
     }
